from django.contrib.postgres.fields import IntegerRangeField
from django.db import models

from .abstract import (
    AbstractUniqueIdentifier,
    AbstractDescriptableResource,
    AbstractValidableResource,
    AbstractEditableResource,
    AlternativeURNs,
)


class Author(
    AbstractUniqueIdentifier,
    AbstractDescriptableResource,
    AbstractEditableResource,
    AbstractValidableResource,
    AlternativeURNs,
    models.Model,
):
    """Author model

    Authors are "historical" authors.

    """

    tlg_id = models.CharField(max_length=20, null=True, blank=True, db_index=True)

    names = models.ManyToManyField(
        "meleager.Name",
        related_name="authors",
        help_text="The multilingual names for this object.",
    )

    city_born = models.ForeignKey(
        "meleager.City",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="authors_born_at",
        related_query_name="%(app_label)s_%(class)ss_city_born",
        help_text="The city where this author was born.",
    )

    born_range_year_date = IntegerRangeField(
        null=True, blank=True, help_text="The date range for the author born date."
    )

    city_died = models.ForeignKey(
        "meleager.City",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="authors_died_at",
        help_text="The city where this author died.",
    )

    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="authors",
        help_text="The images associated to this resource.",
        blank=True,
    )

    died_range_year_date = IntegerRangeField(
        null=True, blank=True, help_text="The date range for the author dead date."
    )

    def __str__(self):
        if self.names.first():
            return f"{self.names.first().name} ({self.pk})"
        else:
            return f"Author PK {self.pk}"
