from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from .abstract import (
    AbstractDescriptableResource,
    AbstractEditableResource,
    AbstractValidableResource,
    AbstractUniqueIdentifier,
    AlternativeURNs,
)


class Scholium(
    AbstractEditableResource,
    AbstractValidableResource,
    AbstractDescriptableResource,
    AbstractUniqueIdentifier,
    AlternativeURNs,
    models.Model,
):
    number = models.IntegerField("Scholium ID", db_index=True, null=True)

    passage = models.ForeignKey(
        "meleager.Passage",
        related_name="scholia",
        on_delete=models.CASCADE,
        null=True,
    )

    cities = models.ManyToManyField(
        "meleager.City",
        related_name="scholia",
    )

    texts = models.ManyToManyField(
        "meleager.Text",
        help_text="Text for this scholium",
        related_name="scholia",
    )

    manuscripts = models.ManyToManyField(
        "meleager.Manuscript",
        help_text="The image of the manuscript for this passage.",
        blank=True,
    )

    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="scholia",
        help_text="The images associated to this resource.",
        blank=True,
    )

    comments = models.ManyToManyField(
        "meleager.Comment",
        related_name="scholia",
        help_text="The comments of this resource.",
        blank=True,
    )

    keywords = models.ManyToManyField(
        "meleager.Keyword",
        related_name="scholia",
        help_text="The keywords that tag this resource.",
        blank=True,
    )

    def get_absolute_url(self):
        return reverse(
            "web:scholium-detail",
            args=(
                self.passage.book.number,
                self.passage.fragment,
                self.passage.sub_fragment,
                self.number,
            ),
        )

    def __str__(self):
        return _("Scholium %(book)s.%(fragment)s%(sub_fragment)s.%(number)s") % {
            "book": self.passage.book.number,
            "fragment": self.passage.fragment,
            "sub_fragment": f".{self.passage.sub_fragment}"
            if self.passage.sub_fragment
            else "",
            "number": self.number,
        }

    class Meta:
        verbose_name_plural = "Scholia"
