import operator
from itertools import groupby

from django import forms

from meleager.models import Author, City, Keyword, Manuscript, Passage, Scholium


class PassageForm(forms.ModelForm):
    fragment = forms.CharField(disabled=True)
    sub_fragment = forms.CharField(disabled=True)

    class Meta:
        model = Passage
        fields = ["fragment", "sub_fragment"]
        exclude = ["urn", "images", "manuscripts", "keywords"]


class AddScholiumForm(forms.ModelForm):
    book = forms.CharField(disabled=True, required=False)
    fragment = forms.CharField(disabled=True, required=False)
    sub_fragment = forms.CharField(disabled=True, required=False)

    class Meta:
        model = Scholium
        fields = ["book", "fragment", "sub_fragment", "number"]


class AddManuscriptForm(forms.ModelForm):
    class Meta:
        model = Manuscript
        fields = ["title", "url"]


class AddAuthorForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        # authors = Author.objects.annotate(
        #     names_list=StringAgg("names__name", delimiter=", ")
        # ).order_by("names__name")

        author_values = (
            Author.objects.all()
            .select_related("names__name")
            .values("pk", "names__name")
        )
        choices += [
            (pk, ", ".join([item["names__name"] for item in group]).strip())
            for pk, group in groupby(author_values, key=operator.itemgetter("pk"))
        ]
        choices = sorted(choices, key=lambda c: c[1])

        self.fields["author"] = forms.ChoiceField(choices=choices)


class AddCityForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        city_values = (
            City.objects.all().select_related("names__name").values("pk", "names__name")
        )
        choices += [
            (pk, ", ".join([item["names__name"] for item in group]))
            for pk, group in groupby(city_values, key=operator.itemgetter("pk"))
        ]
        # cities = City.objects.annotate(
        #     names_list=StringAgg("names__name", delimiter=", ")
        # ).order_by("names__name")
        # choices += cities.values_list("pk", "names_list")
        choices = sorted(choices, key=lambda c: c[1])

        self.fields["city"] = forms.ChoiceField(choices=choices)


class AddKeywordForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = [("", "-------------")]
        # We sort choices, first by `category` then by `name` to be able
        # to propose a consistent optgroup on the HTML side.

        # categories_qs = KeywordCategory.objects.annotate(names_list=StringAgg("names__name", delimiter=", "))
        # categories_qs = categories_qs.order_by("names__name")

        # for c in categories_qs:
        #     choices += [
        #         (
        #             c.names_list,
        #             [
        #                 (keyword.pk, keyword.names_list)
        #                 for keyword in c.keywords.annotate(names_list=StringAgg("names__name", delimiter=", "))
        #             ]
        #         )
        #     ]
        kw_values = Keyword.objects.all().select_related(
            "names__name", "category__names__name"
        )
        kw_values = kw_values.values("pk", "names__name", "category__names__name")

        choices += sorted(
            [
                (
                    cat_name,
                    sorted(
                        [
                            (
                                pk,
                                ", ".join(
                                    sorted([item["names__name"] for item in group])
                                ),
                            )
                            for pk, group in groupby(
                                kw_group, key=operator.itemgetter("pk")
                            )
                        ],
                        key=lambda c: c[1],
                    ),
                )
                for cat_name, kw_group in groupby(
                    kw_values, key=operator.itemgetter("category__names__name")
                )
            ]
        )

        self.fields["keyword"] = forms.ChoiceField(choices=choices)
