from django.test import TestCase
from django.urls import reverse
from meleager.models import Book, Comment, Description, Passage


class ScholiumCommentsTest(TestCase):
    fixtures = ["test_data.json"]

    @classmethod
    def setUpTestData(cls):
        book = Book.objects.first()
        cls.passage = Passage.objects.get(book=book, fragment=12)
        cls.scholium = cls.passage.scholia.first()

    def setUp(self):
        self.client.login(username="editor", password="editor")

    def test_create_comment_form(self):
        response = self.client.get(
            reverse("web:scholium-comment-create", args=(self.scholium.pk,))
        )
        self.assertEquals(response.status_code, 200)

    def test_create_comment(self):
        self.assertEquals(self.scholium.comments.count(), 0)

        self.client.post(
            reverse("web:scholium-comment-create", args=(self.scholium.pk,)),
            data={
                "description": "description",
                "language": "fra",
            },
        )

        first_description = self.scholium.comments.first().descriptions.first()
        self.assertEquals(first_description.description, "description")
        self.assertEquals(first_description.language.code, "fra")

    def test_update_comment_form(self):
        comment = Comment.objects.create()
        self.scholium.comments.add(comment)
        description = Description.objects.create(
            description="old_content", language_id="eng"
        )
        comment.descriptions.add(description)

        response = self.client.get(
            reverse(
                "web:scholium-comment-update",
                kwargs={"comment_pk": comment.pk, "scholium_pk": self.scholium.pk},
            )
        )
        self.assertEquals(response.status_code, 200)

    def test_update_comment(self):
        comment = Comment.objects.create()
        self.scholium.comments.add(comment)

        description = Description.objects.create(
            description="old_content", language_id="eng"
        )
        comment.descriptions.add(description)
        self.assertEquals(comment.descriptions.first().description, "old_content")
        self.assertEquals(comment.descriptions.first().language.code, "eng")

        self.client.post(
            reverse(
                "web:scholium-comment-update",
                kwargs={"comment_pk": comment.pk, "scholium_pk": self.scholium.pk},
            ),
            data={
                "description": "new_content",
                "language": "spa",
            },
        )

        self.assertEquals(comment.descriptions.first().description, "new_content")
        self.assertEquals(comment.descriptions.first().language.code, "spa")

    def test_delete_comment_form(self):
        comment = Comment.objects.create()
        self.scholium.comments.add(comment)
        description = Description.objects.create(
            description="description", language_id="eng"
        )
        comment.descriptions.add(description)

        response = self.client.get(
            reverse(
                "web:scholium-comment-delete",
                kwargs={"comment_pk": comment.pk, "scholium_pk": self.scholium.pk},
            ),
        )
        self.assertEquals(response.status_code, 200)

    def test_delete_comment(self):
        comment = Comment.objects.create()
        description = Description.objects.create(
            description="description", language_id="eng"
        )
        comment.descriptions.add(description)

        self.scholium.comments.add(comment)
        self.assertEquals(self.scholium.comments.count(), 1)

        self.client.post(
            reverse(
                "web:scholium-comment-delete",
                kwargs={"comment_pk": comment.pk, "scholium_pk": self.scholium.pk},
            ),
            data={
                "pk": description.pk,
            },
        )

        self.assertEquals(self.scholium.comments.count(), 0)
