import operator
from itertools import groupby

from django.shortcuts import redirect, reverse
from django.views import generic
from django.utils.translation import gettext as _
from meleager.models import Author, AuthorPassage, Language, Passage

from ..forms.passage import AddAuthorForm
from .mixins import AddContentPermReqMixin, RemoveContentPermReqMixin


class AuthorDetail(generic.DetailView):
    model = Author
    template_name = "web/author/detail.html"

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.prefetch_related(
            "names", "names__language", "passages", "passages__book"
        )
        return qs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        author = context["object"]
        context["author"] = author.names.order_by("name").values(
            "name", "language__code"
        )
        context["page_title"] = ", ".join(elem["name"] for elem in context["author"])

        context["passages"] = [
            {
                "pk": elem["pk"],
                "short_title": "{}.{}{}".format(
                    elem["book__number"], elem["fragment"], elem["sub_fragment"]
                ),
                "url": reverse(
                    "web:passage-detail",
                    args=(elem["book__number"], elem["fragment"], elem["sub_fragment"]),
                ),
            }
            for elem in author.passages.values(
                "pk", "book__number", "fragment", "sub_fragment"
            )
        ]

        return context


class AuthorList(generic.ListView):
    model = Author
    template_name = "web/author/list.html"
    # context_object_name = "authors"

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.prefetch_related("names__name")
        qs = qs.order_by("names__name").values("pk", "names__name")
        return qs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["page_title"] = _("Authors")
        context["authors"] = {
            pk: [elem["names__name"] for elem in items]
            for pk, items in groupby(context["object_list"], operator.itemgetter("pk"))
        }
        return context


class PassageAuthorCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Passage
    pk_url_kwarg = "passage_pk"
    form_class = AddAuthorForm
    template_name = "web/author/author_create.html"

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        author_pk = form.cleaned_data.get("author")
        AuthorPassage.objects.create(
            author_id=author_pk, passage=self.object, user=self.request.user
        )
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:passage-author-create",
            kwargs={"passage_pk": self.kwargs.get("passage_pk")},
        )
        return context

    def get_success_url(self):
        return f"{self.get_object().get_absolute_url()}#authors"


class PassageAuthorDelete(RemoveContentPermReqMixin, generic.DeleteView):
    model = Author
    pk_url_kwarg = "author_pk"
    template_name = "web/author/author_delete.html"
    context_object_name = "author"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:passage-author-delete",
            kwargs={
                "passage_pk": self.kwargs.get("passage_pk"),
                "author_pk": self.kwargs.get("author_pk"),
            },
        )

        return context

    def get_success_url(self):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        return f"{passage.get_absolute_url()}#authors"
