import operator
from itertools import groupby

from django.shortcuts import reverse
from django.utils.translation import gettext as _
from django.views import generic
from meleager.models import City, Language, Passage, Scholium
from web.forms.passage import AddCityForm

from .mixins import AddContentPermReqMixin, RemoveContentPermReqMixin


class CityList(generic.ListView):
    model = City
    template_name = "web/city/list.html"
    context_object_name = "cities"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["page_title"] = _("Cities")
        cities = context["cities"]
        city_values = cities.select_related("names__name").values("pk", "names__name")
        cities_pk_names = [
            (pk, ", ".join([item["names__name"] for item in group]))
            for pk, group in groupby(city_values, key=operator.itemgetter("pk"))
        ]
        cities_pk_names = sorted(cities_pk_names, key=lambda c: c[1])
        context["cities_pk_names"] = cities_pk_names
        return context


class CityDetail(generic.DetailView):
    model = City
    template_name = "web/city/detail.html"
    context_object_name = "city"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        city = context["city"]
        context["page_title"] = ", ".join(
            sorted([name.name for name in city.names.all()])
        )
        context["passages"] = city.passages.all()
        context["scholia"] = city.scholia.all()
        context["authors_born_at"] = city.authors_born_at.all()
        context["authors_died_at"] = city.authors_died_at.all()
        return context


class PassageCityCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Passage
    pk_url_kwarg = "passage_pk"
    form_class = AddCityForm
    template_name = "web/city/city_create.html"

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:passage-city-create",
            kwargs={"passage_pk": self.kwargs.get("passage_pk")},
        )
        return context

    def form_valid(self, form):
        passage = self.object
        city_pk = form.cleaned_data.get("city")
        city = City.objects.get(pk=city_pk)
        passage.cities.add(city)
        return super().form_valid(form)

    def get_success_url(self):
        return f"{self.get_object().get_absolute_url()}#cities"


class PassageCityDelete(RemoveContentPermReqMixin, generic.DeleteView):
    model = City
    template_name = "web/city/city_delete.html"
    pk_url_kwarg = "city_pk"
    context_object_name = "city"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:passage-city-delete",
            kwargs={
                "passage_pk": self.kwargs.get("passage_pk"),
                "city_pk": self.kwargs.get("city_pk"),
            },
        )

        return context

    def get_success_url(self):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        return f"{passage.get_absolute_url()}#cities"


class ScholiumCityCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Scholium
    pk_url_kwarg = "scholium_pk"
    form_class = AddCityForm
    template_name = "web/city/city_create.html"

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:scholium-city-create",
            kwargs={"scholium_pk": self.kwargs.get("scholium_pk")},
        )
        return context

    def form_valid(self, form):
        city_pk = form.cleaned_data.get("city")
        city = City.objects.get(pk=city_pk)
        self.get_object().cities.add(city)
        return super().form_valid(form)

    def get_success_url(self):
        return f"{self.get_object().get_absolute_url()}#cities"


class ScholiumCityDelete(RemoveContentPermReqMixin, generic.DeleteView):
    model = City
    template_name = "web/city/city_delete.html"
    pk_url_kwarg = "city_pk"
    context_object_name = "city"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:scholium-city-delete",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "city_pk": self.kwargs.get("city_pk"),
            },
        )

        return context

    def get_success_url(self):
        scholium = Scholium.objects.get(pk=self.kwargs.get("scholium_pk"))
        return f"{scholium.get_absolute_url()}#cities"
