from django.shortcuts import get_object_or_404, redirect, reverse
from django.views import generic
from meleager.models import Comment, Description, Passage, Scholium
from web.forms.comment import CommentForm
from web.forms.comment_description import CommentCreateForm, CommentUpdateForm
from web.forms.description import DescriptionForm

from .mixins import AddContentPermReqMixin, RemoveContentPermReqMixin


class PassageCommentCreate(AddContentPermReqMixin, generic.FormView):
    model = Comment
    form_class = CommentCreateForm
    template_name = "web/comment/comment_create.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["passage"] = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        context["create_url"] = reverse(
            "web:passage-comment-create",
            kwargs={"passage_pk": self.kwargs.get("passage_pk")},
        )
        return context

    def post(self, *args, **kwargs):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        description_form = DescriptionForm(self.request.POST)
        comment_form = CommentForm(self.request.POST)

        if description_form.is_valid() and comment_form.is_valid():
            description = description_form.save()
            comment = comment_form.save()
            comment.descriptions.add(description)
            passage.comments.add(comment)
            return redirect(f"{passage.get_absolute_url()}#comment-{comment.pk}")


class PassageCommentUpdate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Comment
    form_class = CommentUpdateForm
    template_name = "web/comment/comment_update.html"
    pk_url_kwarg = "comment_pk"

    def get_initial(self, **kwargs):
        comment = self.get_object()
        # TODO: handle multiple descriptions?
        description = comment.descriptions.first()
        return {
            "pk": comment.pk,
            "description": description.description,
            "language": description.language,
        }

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["passage"] = Passage.objects.get(pk=self.kwargs.get("passage_pk"))

        context["update_url"] = reverse(
            "web:passage-comment-update",
            kwargs={
                "passage_pk": self.kwargs.get("passage_pk"),
                "comment_pk": self.kwargs.get("comment_pk"),
            },
        )
        return context

    def form_valid(self, form):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        comment_pk = self.kwargs.get("comment_pk")
        comment = get_object_or_404(Comment, pk=comment_pk)
        # TODO: handle multiple descriptions?
        description = comment.descriptions.first()
        description.description = form.cleaned_data["description"]
        description.language = form.cleaned_data["language"]
        description.save()
        return redirect(f"{passage.get_absolute_url()}#comment-{comment_pk}")


class PassageCommentDelete(AddContentPermReqMixin, generic.DeleteView):
    model = Comment
    template_name = "web/comment/comment_delete.html"
    pk_url_kwarg = "comment_pk"
    context_object_name = "comment"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:passage-comment-delete",
            kwargs={
                "passage_pk": self.kwargs.get("passage_pk"),
                "comment_pk": self.kwargs.get("comment_pk"),
            },
        )

        return context

    def form_valid(self, form):
        self.get_object().descriptions.delete()
        return super().form_valid(form)

    def get_success_url(self):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        return f"{passage.get_absolute_url()}#comments"


class ScholiumCommentCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Scholium
    form_class = CommentCreateForm
    template_name = "web/comment/comment_create.html"
    pk_url_kwarg = "scholium_pk"
    context_object_name = "scholium"

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:scholium-comment-create",
            kwargs={"scholium_pk": self.kwargs.get("scholium_pk")},
        )
        return context

    def post(self, *args, **kwargs):
        scholium = self.get_object()
        description_form = DescriptionForm(self.request.POST)
        comment_form = CommentForm(self.request.POST)

        if description_form.is_valid() and comment_form.is_valid():
            description = description_form.save()
            comment = comment_form.save()
            comment.descriptions.add(description)
            scholium.comments.add(comment)
            return redirect(f"{scholium.get_absolute_url()}#comment-{comment.pk}")


class ScholiumCommentUpdate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Comment
    form_class = CommentUpdateForm
    template_name = "web/comment/comment_update.html"
    pk_url_kwarg = "comment_pk"

    def get_initial(self, **kwargs):
        comment = self.get_object()
        # TODO: handle multiple descriptions?
        description = comment.descriptions.first()
        return {
            "pk": comment.pk,
            "description": description.description,
            "language": description.language,
        }

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["scholium"] = self.get_object().scholia.first()
        context["update_url"] = reverse(
            "web:scholium-comment-update",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "comment_pk": self.kwargs.get("comment_pk"),
            },
        )
        return context

    def form_valid(self, form):
        scholium = self.get_object().scholia.first()
        comment_pk = self.kwargs.get("comment_pk")
        comment = get_object_or_404(Comment, pk=comment_pk)
        # TODO: handle multiple descriptions?
        description = comment.descriptions.first()
        description.description = form.cleaned_data["description"]
        description.language = form.cleaned_data["language"]
        description.save()
        return redirect(f"{scholium.get_absolute_url()}#comment-{comment_pk}")


class ScholiumCommentDelete(RemoveContentPermReqMixin, generic.DeleteView):
    model = Comment
    template_name = "web/comment/comment_delete.html"
    pk_url_kwarg = "comment_pk"
    context_object_name = "comment"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:scholium-comment-delete",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "comment_pk": self.kwargs.get("comment_pk"),
            },
        )

        return context

    def form_valid(self, form):
        self.get_object().descriptions.delete()
        return super().form_valid(form)

    def get_success_url(self):
        scholium = Scholium.objects.get(pk=self.kwargs.get("scholium_pk"))
        return f"{scholium.get_absolute_url()}#comments"
