import operator
from itertools import groupby

from django.shortcuts import get_object_or_404, redirect, reverse
from django.views import generic
from django.utils.translation import gettext as _

from meleager.models import Keyword, Passage, Scholium

from ..forms.passage import AddKeywordForm
from .mixins import AddContentPermReqMixin, RemoveContentPermReqMixin


class KeywordList(generic.ListView):
    model = Keyword
    template_name = "web/keyword/list.html"
    context_object_name = "keywords"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        keywords = context["keywords"]
        context["page_title"] = _("Keywords")
        kw_values = keywords.select_related("names__name", "category__names__name")
        kw_values = kw_values.values("pk", "names__name", "category__names__name")
        keywords_cat_pk_names = sorted(
            [
                (
                    cat_name,
                    sorted(
                        [
                            (
                                pk,
                                ", ".join(
                                    sorted([item["names__name"] for item in group])
                                ),
                            )
                            for pk, group in groupby(
                                kw_group, key=operator.itemgetter("pk")
                            )
                        ],
                        key=lambda c: c[1],
                    ),
                )
                for cat_name, kw_group in groupby(
                    kw_values, key=operator.itemgetter("category__names__name")
                )
            ]
        )

        context["keywords_cat_pk_names"] = keywords_cat_pk_names
        return context


class KeywordDetail(generic.DetailView):
    model = Keyword
    template_name = "web/keyword/detail.html"
    context_object_name = "keyword"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        keyword = context["keyword"]
        context["page_title"] = ", ".join(
            sorted([name.name for name in keyword.names.all()])
        )
        context["passages"] = keyword.passages.all()
        context["scholia"] = keyword.scholia.all()
        return context


class PassageKeywordCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Passage
    pk_url_kwarg = "passage_pk"
    form_class = AddKeywordForm
    template_name = "web/keyword/keyword_create.html"

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        keyword_pk = form.cleaned_data.get("keyword")
        keyword = Keyword.objects.get(pk=keyword_pk)
        # TODO: link the attached object to the current user.
        self.object.keywords.add(keyword)
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:passage-keyword-create",
            kwargs={"passage_pk": self.kwargs.get("passage_pk")},
        )
        return context

    def get_success_url(self):
        return f"{self.object.get_absolute_url()}#keywords"


class PassageKeywordDelete(RemoveContentPermReqMixin, generic.DetailView):
    model = Keyword
    template_name = "web/keyword/keyword_delete.html"
    pk_url_kwarg = "keyword_pk"
    context_object_name = "keyword"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:passage-keyword-delete",
            kwargs={
                "passage_pk": self.kwargs.get("passage_pk"),
                "keyword_pk": self.kwargs.get("keyword_pk"),
            },
        )

        return context

    def post(self, request, *args, **kwargs):
        passage = Passage.objects.get(pk=self.kwargs.get("passage_pk"))
        keyword = Keyword.objects.get(pk=self.kwargs.get("keyword_pk"))
        passage.keywords.remove(keyword)
        return redirect(f"{passage.get_absolute_url()}#keywords")


class ScholiumKeywordCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Scholium
    pk_url_kwarg = "scholium_pk"
    form_class = AddKeywordForm
    template_name = "web/keyword/keyword_create.html"

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        keyword_pk = form.cleaned_data.get("keyword")
        keyword = Keyword.objects.get(pk=keyword_pk)
        # TODO: link the attached object to the current user.
        self.object.keywords.add(keyword)
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:scholium-keyword-create",
            kwargs={"scholium_pk": self.kwargs.get("scholium_pk")},
        )
        return context

    def get_success_url(self):
        return f"{self.object.get_absolute_url()}#keywords"


class ScholiumKeywordDelete(RemoveContentPermReqMixin, generic.DetailView):
    model = Keyword
    template_name = "web/keyword/keyword_delete.html"
    pk_url_kwarg = "keyword_pk"
    context_object_name = "keyword"

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["delete_url"] = reverse(
            "web:scholium-keyword-delete",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "keyword_pk": self.kwargs.get("keyword_pk"),
            },
        )

        return context

    def post(self, request, *args, **kwargs):
        scholium = Scholium.objects.get(pk=self.kwargs.get("scholium_pk"))
        keyword = Keyword.objects.get(pk=self.kwargs.get("keyword_pk"))
        scholium.keywords.remove(keyword)
        return redirect(f"{scholium.get_absolute_url()}#keywords")
