from django.shortcuts import get_object_or_404, reverse
from django.views import generic
from django.utils.translation import gettext as _
from meleager.models import Language, Passage, Scholium

from ..forms.passage import AddScholiumForm
from .mixins import AddContentPermReqMixin


class ScholiumCreate(
    AddContentPermReqMixin, generic.detail.SingleObjectMixin, generic.FormView
):
    model = Passage
    form_class = AddScholiumForm
    template_name = "web/scholium/scholium_create.html"
    context_object_name = "passage"
    pk_url_kwarg = "passage_pk"

    def get_initial(self, **kwargs):
        passage = self.get_object()
        max_number = max(
            (scholium.number for scholium in passage.scholia.all()), default=1
        )
        return {
            "number": max_number + 1,
            "book": passage.book.number,
            "fragment": passage.fragment,
            "sub_fragment": passage.sub_fragment,
        }

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["page_title"] = _("Create Scholium for %(passage)s") % {
            "passage": self.object,
        }
        context["create_url"] = reverse(
            "web:scholium-create",
            kwargs={"passage_pk": self.kwargs.get("passage_pk")},
        )
        return context

    def form_valid(self, form):
        passage = self.get_object()
        number = form.cleaned_data.get("number")
        scholium = Scholium.objects.create(number=number)
        passage.scholia.add(scholium)
        return super().form_valid(form)

    def get_success_url(self):
        return f"{self.get_object().get_absolute_url()}#scholia"


class ScholiumDetail(generic.DetailView):
    model = Scholium
    template_name = "web/scholium/detail.html"
    context_object_name = "scholium"

    def get_object(self):
        book = self.kwargs.get("book")
        fragment = self.kwargs.get("fragment")
        sub_fragment = self.kwargs.get("sub_fragment")
        number = self.kwargs.get("number")

        return get_object_or_404(
            Scholium,
            passage__book__number=book,
            passage__fragment=fragment,
            passage__sub_fragment=sub_fragment,
            number=number,
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        scholium = self.get_object()
        keywords = scholium.keywords.all()
        context.update(
            {
                "keywords": keywords,
                "keywords_categories": {keyword.category for keyword in keywords},
                "book": scholium.passage.book,
                "passage": scholium.passage,
                "cities": scholium.cities.all(),
                "page_title": str(scholium),
            }
        )
        return context
