from django.shortcuts import reverse, redirect
from django.views import generic
from meleager.models import Description, Scholium

from .mixins import (
    AddContentPermReqMixin,
    DescriptionCreateMixin,
    RemoveContentPermReqMixin,
)


class ScholiumDescriptionCreate(DescriptionCreateMixin):
    model = Scholium
    context_object_name = "scholium"
    pk_url_kwarg = "scholium_pk"

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        context = super().get_context_data(**kwargs)
        context["create_url"] = reverse(
            "web:scholium-description-create",
            kwargs={"scholium_pk": self.kwargs.get("scholium_pk")},
        )
        return context

    def form_valid(self, form):
        self.new_object = self.create_associate_from_form(form)
        return super().form_valid(form)

    def get_success_url(self):
        return (
            f"{self.get_object().get_absolute_url()}#description-{self.new_object.pk}"
        )


class ScholiumDescriptionUpdate(AddContentPermReqMixin, generic.UpdateView):
    model = Description
    fields = ("description", "language")
    template_name = "web/description/description_update.html"
    pk_url_kwarg = "description_pk"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["scholium"] = self.get_object().meleager_scholium.first()
        context["update_url"] = reverse(
            "web:scholium-description-update",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "description_pk": self.kwargs.get("description_pk"),
            },
        )
        return context

    def get_success_url(self):
        scholium = Scholium.objects.get(pk=self.kwargs.get("scholium_pk"))
        return f"{scholium.get_absolute_url()}#description-{self.get_object().pk}"


class ScholiumDescriptionDelete(RemoveContentPermReqMixin, generic.DetailView):
    model = Description
    template_name = "web/description/description_delete.html"
    pk_url_kwarg = "description_pk"
    context_object_name = "description"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["scholium"] = self.get_object().meleager_scholium.first()
        context["delete_url"] = reverse(
            "web:scholium-description-delete",
            kwargs={
                "scholium_pk": self.kwargs.get("scholium_pk"),
                "description_pk": self.kwargs.get("description_pk"),
            },
        )

        return context

    def post(self, request, *args, **kwargs):
        scholium = self.get_object().meleager_scholium.first()
        self.get_object().delete()
        return redirect(f"{scholium.get_absolute_url()}#descriptions")
